import argparse
import collections


class CommandLineParser:
    def __init__(self, arguments, validator):
        parser = argparse.ArgumentParser(description="Parse input command line")
        parser.add_argument('--current-version', nargs=1, help='Current version (upgrade is made from)')
        parser.add_argument('--target-version', nargs=1, help='Target version (upgrade is made to)')
        parser.add_argument('--rootfs', nargs=1, help='Path to target rootfs')
        parser.add_argument('--patches-path', nargs=1, help='Path to upgrade patches')
        parser.add_argument('--clean', action='store_true', help='Clean upgrade')
        self.__params = parser.parse_args(arguments)

        if not validator(self.__params):
            raise AttributeError("Base arguments are missed")

    @staticmethod
    def __version_string_to_structure(version):
        block = version.split('.')
        if len(block) != 3 or not block[0].isdigit() or not block[1].isdigit() or not block[2].isdigit():
            raise AttributeError("Wrong version format. Should be X.X.X")

        version_structure = collections.namedtuple("version_structure", "major minor build")
        return version_structure(major=int(block[0]), minor=int(block[1]), build=int(block[2]))

    @staticmethod
    def __version_structure_to_string(version):
        return "{0}.{1}.{2}".format(version.major, version.minor, version.build)

    def current_version(self):
        return self.__params.current_version[0]

    def target_version(self):
        return self.__params.target_version[0]

    def rootfs_path(self):
        return self.__params.rootfs[0]

    def patches_path(self):
        return self.__params.patches_path[0]

    def is_upgrade(self):
        current = self.__version_string_to_structure(self.__params.current_version[0])
        target = self.__version_string_to_structure(self.__params.target_version[0])

        if current.major == target.major:
            if current.minor == target.minor:
                return current.build < target.build
            return current.minor < target.minor
        return current.major < target.major

    def is_clean(self):
        return self.__params.clean
